@@ -1,54 +1,49 @@
 use std::{
-    cell::RefCell, error::Error, fmt, future::Future, marker, net, pin::Pin, rc::Rc,
-    task,
+    cell::RefCell, error::Error, fmt, future::Future, marker, pin::Pin, rc::Rc, task,
 };
 
-use crate::codec::{AsyncRead, AsyncWrite};
-use crate::framed::State as IoState;
 use crate::http::body::MessageBody;
 use crate::http::config::{DispatcherConfig, OnRequest, ServiceConfig};
 use crate::http::error::{DispatchError, ResponseError};
-use crate::http::helpers::DataFactory;
 use crate::http::request::Request;
 use crate::http::response::Response;
-use crate::service::{pipeline_factory, IntoServiceFactory, Service, ServiceFactory};
-use crate::{rt::net::TcpStream, time::Millis, util::Pool};
+use crate::io::{types, Filter, Io};
+use crate::service::{IntoServiceFactory, Service, ServiceFactory};
+use crate::time::Millis;
 
 use super::codec::Codec;
 use super::dispatcher::Dispatcher;
 use super::{ExpectHandler, UpgradeHandler};
 
 /// `ServiceFactory` implementation for HTTP1 transport
-pub struct H1Service<T, S, B, X = ExpectHandler, U = UpgradeHandler<T>> {
+pub struct H1Service<F, S, B, X = ExpectHandler, U = UpgradeHandler<F>> {
     srv: S,
     cfg: ServiceConfig,
     expect: X,
     upgrade: Option<U>,
-    on_connect: Option<Rc<dyn Fn(&T) -> Box<dyn DataFactory>>>,
-    on_request: RefCell<Option<OnRequest<T>>>,
+    on_request: RefCell<Option<OnRequest>>,
     #[allow(dead_code)]
     handshake_timeout: Millis,
-    _t: marker::PhantomData<(T, B)>,
+    _t: marker::PhantomData<(F, B)>,
 }
 
-impl<T, S, B> H1Service<T, S, B>
+impl<F, S, B> H1Service<F, S, B>
 where
-    S: ServiceFactory<Config = (), Request = Request>,
+    S: ServiceFactory<Request>,
     S::Error: ResponseError + 'static,
     S::InitError: fmt::Debug,
     S::Response: Into<Response<B>>,
     B: MessageBody,
 {
     /// Create new `HttpService` instance with config.
-    pub(crate) fn with_config<F: IntoServiceFactory<S>>(
+    pub(crate) fn with_config<U: IntoServiceFactory<S, Request>>(
         cfg: ServiceConfig,
-        service: F,
+        service: U,
     ) -> Self {
         H1Service {
             srv: service.into_factory(),
             expect: ExpectHandler,
             upgrade: None,
-            on_connect: None,
             on_request: RefCell::new(None),
             handshake_timeout: cfg.0.ssl_handshake_timeout,
             _t: marker::PhantomData,
@@ -57,80 +52,37 @@ where
     }
 }
 
-impl<S, B, X, U> H1Service<TcpStream, S, B, X, U>
-where
-    S: ServiceFactory<Config = (), Request = Request>,
-    S::Error: ResponseError + 'static,
-    S::InitError: fmt::Debug,
-    S::Response: Into<Response<B>>,
-    S::Future: 'static,
-    B: MessageBody,
-    X: ServiceFactory<Config = (), Request = Request, Response = Request>,
-    X::Error: ResponseError + 'static,
-    X::InitError: fmt::Debug,
-    X::Future: 'static,
-    U: ServiceFactory<
-        Config = (),
-        Request = (Request, TcpStream, IoState, Codec),
-        Response = (),
-    >,
-    U::Error: fmt::Display + Error + 'static,
-    U::InitError: fmt::Debug,
-    U::Future: 'static,
-{
-    /// Create simple tcp stream service
-    pub fn tcp(
-        self,
-    ) -> impl ServiceFactory<
-        Config = (),
-        Request = TcpStream,
-        Response = (),
-        Error = DispatchError,
-        InitError = (),
-    > {
-        pipeline_factory(|io: TcpStream| async move {
-            let peer_addr = io.peer_addr().ok();
-            Ok((io, peer_addr))
-        })
-        .and_then(self)
-    }
-}
-
 #[cfg(feature = "openssl")]
 mod openssl {
+    use ntex_tls::openssl::{Acceptor, SslFilter};
+    use tls_openssl::ssl::SslAcceptor;
+
     use super::*;
+    use crate::{server::SslError, service::pipeline_factory};
 
-    use crate::server::openssl::{Acceptor, SslAcceptor, SslStream};
-    use crate::server::SslError;
-
-    impl<S, B, X, U> H1Service<SslStream<TcpStream>, S, B, X, U>
+    impl<F, S, B, X, U> H1Service<SslFilter<F>, S, B, X, U>
     where
-        S: ServiceFactory<Config = (), Request = Request>,
+        F: Filter,
+        S: ServiceFactory<Request>,
         S::Error: ResponseError + 'static,
         S::InitError: fmt::Debug,
         S::Response: Into<Response<B>>,
         S::Future: 'static,
         B: MessageBody,
-        X: ServiceFactory<Config = (), Request = Request, Response = Request>,
+        X: ServiceFactory<Request, Response = Request>,
         X::Error: ResponseError + 'static,
         X::InitError: fmt::Debug,
         X::Future: 'static,
-        U: ServiceFactory<
-            Config = (),
-            Request = (Request, SslStream<TcpStream>, IoState, Codec),
-            Response = (),
-        >,
-        U::Error: fmt::Display + Error + 'static,
+        U: ServiceFactory<(Request, Io<SslFilter<F>>, Codec), Response = ()> + 'static,
+        U::Error: fmt::Display + Error,
         U::InitError: fmt::Debug,
-        U::Future: 'static,
     {
         /// Create openssl based service
         pub fn openssl(
             self,
             acceptor: SslAcceptor,
         ) -> impl ServiceFactory<
-            Config = (),
-            Request = TcpStream,
+            Io<F>,
             Response = (),
             Error = SslError<DispatchError>,
             InitError = (),
@@ -141,10 +93,6 @@ mod openssl {
                     .map_err(SslError::Ssl)
                     .map_init_err(|_| panic!()),
             )
-            .and_then(|io: SslStream<TcpStream>| async move {
-                let peer_addr = io.get_ref().peer_addr().ok();
-                Ok((io, peer_addr))
-            })
             .and_then(self.map_err(SslError::Service))
         }
     }
@@ -152,70 +100,65 @@ mod openssl {
 
 #[cfg(feature = "rustls")]
 mod rustls {
-    use super::*;
-    use crate::server::rustls::{Acceptor, ServerConfig, TlsStream};
-    use crate::server::SslError;
     use std::fmt;
 
-    impl<S, B, X, U> H1Service<TlsStream<TcpStream>, S, B, X, U>
+    use ntex_tls::rustls::{Acceptor, TlsFilter};
+    use tls_rustls::ServerConfig;
+
+    use super::*;
+    use crate::{server::SslError, service::pipeline_factory};
+
+    impl<F, S, B, X, U> H1Service<TlsFilter<F>, S, B, X, U>
     where
-        S: ServiceFactory<Config = (), Request = Request>,
+        F: Filter,
+        S: ServiceFactory<Request>,
         S::Error: ResponseError + 'static,
         S::InitError: fmt::Debug,
         S::Response: Into<Response<B>>,
         S::Future: 'static,
         B: MessageBody,
-        X: ServiceFactory<Config = (), Request = Request, Response = Request>,
+        X: ServiceFactory<Request, Response = Request>,
         X::Error: ResponseError + 'static,
         X::InitError: fmt::Debug,
         X::Future: 'static,
-        U: ServiceFactory<
-            Config = (),
-            Request = (Request, TlsStream<TcpStream>, IoState, Codec),
-            Response = (),
-        >,
-        U::Error: fmt::Display + Error + 'static,
+        U: ServiceFactory<(Request, Io<TlsFilter<F>>, Codec), Response = ()> + 'static,
+        U::Error: fmt::Display + Error,
         U::InitError: fmt::Debug,
-        U::Future: 'static,
     {
         /// Create rustls based service
         pub fn rustls(
             self,
             config: ServerConfig,
         ) -> impl ServiceFactory<
-            Config = (),
-            Request = TcpStream,
+            Io<F>,
             Response = (),
             Error = SslError<DispatchError>,
             InitError = (),
         > {
             pipeline_factory(
-                Acceptor::new(config)
+                Acceptor::from(config)
                     .timeout(self.handshake_timeout)
-                    .map_err(SslError::Ssl)
+                    .map_err(|e| SslError::Ssl(Box::new(e)))
                     .map_init_err(|_| panic!()),
             )
-            .and_then(|io: TlsStream<TcpStream>| async move {
-                let peer_addr = io.get_ref().0.peer_addr().ok();
-                Ok((io, peer_addr))
-            })
             .and_then(self.map_err(SslError::Service))
         }
     }
 }
 
-impl<T, S, B, X, U> H1Service<T, S, B, X, U>
+impl<F, S, B, X, U> H1Service<F, S, B, X, U>
 where
-    S: ServiceFactory<Config = (), Request = Request>,
+    F: Filter,
+    S: ServiceFactory<Request>,
     S::Error: ResponseError + 'static,
     S::Response: Into<Response<B>>,
     S::InitError: fmt::Debug,
     S::Future: 'static,
     B: MessageBody,
 {
-    pub fn expect<X1>(self, expect: X1) -> H1Service<T, S, B, X1, U>
+    pub fn expect<X1>(self, expect: X1) -> H1Service<F, S, B, X1, U>
     where
-        X1: ServiceFactory<Request = Request, Response = Request>,
+        X1: ServiceFactory<Request, Response = Request>,
         X1::Error: ResponseError + 'static,
         X1::InitError: fmt::Debug,
         X1::Future: 'static,
@@ -225,16 +168,15 @@ where
             cfg: self.cfg,
             srv: self.srv,
             upgrade: self.upgrade,
-            on_connect: self.on_connect,
             on_request: self.on_request,
             handshake_timeout: self.handshake_timeout,
             _t: marker::PhantomData,
         }
     }
 
-    pub fn upgrade<U1>(self, upgrade: Option<U1>) -> H1Service<T, S, B, X, U1>
+    pub fn upgrade<U1>(self, upgrade: Option<U1>) -> H1Service<F, S, B, X, U1>
     where
-        U1: ServiceFactory<Request = (Request, T, IoState, Codec), Response = ()>,
+        U1: ServiceFactory<(Request, Io<F>, Codec), Response = ()>,
         U1::Error: fmt::Display + Error + 'static,
         U1::InitError: fmt::Debug,
         U1::Future: 'static,
@@ -244,66 +186,48 @@ where
             cfg: self.cfg,
             srv: self.srv,
             expect: self.expect,
-            on_connect: self.on_connect,
             on_request: self.on_request,
             handshake_timeout: self.handshake_timeout,
             _t: marker::PhantomData,
         }
     }
 
-    /// Set on connect callback.
-    pub(crate) fn on_connect(
-        mut self,
-        f: Option<Rc<dyn Fn(&T) -> Box<dyn DataFactory>>>,
-    ) -> Self {
-        self.on_connect = f;
-        self
-    }
-
     /// Set req request callback.
     ///
     /// It get called once per request.
-    pub(crate) fn on_request(self, f: Option<OnRequest<T>>) -> Self {
+    pub(crate) fn on_request(self, f: Option<OnRequest>) -> Self {
         *self.on_request.borrow_mut() = f;
         self
     }
 }
 
-impl<T, S, B, X, U> ServiceFactory for H1Service<T, S, B, X, U>
+impl<F, S, B, X, U> ServiceFactory<Io<F>> for H1Service<F, S, B, X, U>
 where
-    T: AsyncRead + AsyncWrite + Unpin + 'static,
-    S: ServiceFactory<Config = (), Request = Request>,
+    F: Filter + 'static,
+    S: ServiceFactory<Request>,
     S::Error: ResponseError + 'static,
     S::Response: Into<Response<B>>,
     S::InitError: fmt::Debug,
     S::Future: 'static,
     B: MessageBody,
-    X: ServiceFactory<Config = (), Request = Request, Response = Request>,
+    X: ServiceFactory<Request, Response = Request>,
     X::Error: ResponseError + 'static,
     X::InitError: fmt::Debug,
     X::Future: 'static,
-    U: ServiceFactory<
-        Config = (),
-        Request = (Request, T, IoState, Codec),
-        Response = (),
-    >,
-    U::Error: fmt::Display + Error + 'static,
+    U: ServiceFactory<(Request, Io<F>, Codec), Response = ()> + 'static,
+    U::Error: fmt::Display + Error,
     U::InitError: fmt::Debug,
-    U::Future: 'static,
 {
-    type Config = ();
-    type Request = (T, Option<net::SocketAddr>);
     type Response = ();
     type Error = DispatchError;
     type InitError = ();
-    type Service = H1ServiceHandler<T, S::Service, B, X::Service, U::Service>;
+    type Service = H1ServiceHandler<F, S::Service, B, X::Service, U::Service>;
     type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
         let fut = self.srv.new_service(());
         let fut_ex = self.expect.new_service(());
         let fut_upg = self.upgrade.as_ref().map(|f| f.new_service(()));
-        let on_connect = self.on_connect.clone();
         let on_request = self.on_request.borrow_mut().take();
         let cfg = self.cfg.clone();
 
@@ -326,12 +250,9 @@ where
             let config = Rc::new(DispatcherConfig::new(
                 cfg, service, expect, upgrade, on_request,
             ));
-            let pool = config.pool.into();
 
             Ok(H1ServiceHandler {
-                pool,
                 config,
-                on_connect,
                 _t: marker::PhantomData,
             })
         })
@@ -339,29 +260,26 @@ where
 }
 
 /// `Service` implementation for HTTP1 transport
-pub struct H1ServiceHandler<T, S: Service, B, X: Service, U: Service> {
-    pool: Pool,
-    config: Rc<DispatcherConfig<T, S, X, U>>,
-    on_connect: Option<Rc<dyn Fn(&T) -> Box<dyn DataFactory>>>,
-    _t: marker::PhantomData<(T, B)>,
+pub struct H1ServiceHandler<F, S, B, X, U> {
+    config: Rc<DispatcherConfig<S, X, U>>,
+    _t: marker::PhantomData<(F, B)>,
 }
 
-impl<T, S, B, X, U> Service for H1ServiceHandler<T, S, B, X, U>
+impl<F, S, B, X, U> Service<Io<F>> for H1ServiceHandler<F, S, B, X, U>
 where
-    T: AsyncRead + AsyncWrite + Unpin + 'static,
-    S: Service<Request = Request>,
+    F: Filter + 'static,
+    S: Service<Request>,
     S::Error: ResponseError + 'static,
     S::Response: Into<Response<B>>,
     B: MessageBody,
-    X: Service<Request = Request, Response = Request>,
+    X: Service<Request, Response = Request>,
     X::Error: ResponseError + 'static,
-    U: Service<Request = (Request, T, IoState, Codec), Response = ()>,
-    U::Error: fmt::Display + Error + 'static,
+    U: Service<(Request, Io<F>, Codec), Response = ()> + 'static,
+    U::Error: fmt::Display + Error,
 {
-    type Request = (T, Option<net::SocketAddr>);
     type Response = ();
     type Error = DispatchError;
-    type Future = Dispatcher<T, S, B, X, U>;
+    type Future = Dispatcher<F, S, B, X, U>;
 
     fn poll_ready(
         &self,
@@ -400,8 +318,6 @@ where
             ready
         };
 
-        let ready = self.pool.poll_ready(cx).is_ready() && ready;
-
         if ready {
             task::Poll::Ready(Ok(()))
         } else {
@@ -409,11 +325,7 @@ where
         }
     }
 
-    fn poll_shutdown(
-        &self,
-        cx: &mut task::Context<'_>,
-        is_error: bool,
-    ) -> task::Poll<()> {
+    fn poll_shutdown(&self, cx: &mut task::Context<'_>, is_error: bool) -> task::Poll<()> {
         let ready = self.config.expect.poll_shutdown(cx, is_error).is_ready();
         let ready = self.config.service.poll_shutdown(cx, is_error).is_ready() && ready;
         let ready = if let Some(ref upg) = self.config.upgrade {
@@ -429,9 +341,12 @@ where
         }
     }
 
-    fn call(&self, (io, addr): Self::Request) -> Self::Future {
-        let on_connect = self.on_connect.as_ref().map(|f| f(&io));
+    fn call(&self, io: Io<F>) -> Self::Future {
+        log::trace!(
+            "New http1 connection, peer address {:?}",
+            io.query::<types::PeerAddr>().get()
+        );
 
-        Dispatcher::new(io, self.config.clone(), addr, on_connect)
+        Dispatcher::new(io, self.config.clone())
     }
 }
