@@ -1,70 +1,67 @@
 use std::task::{Context, Poll};
-use std::{
-    convert::TryFrom, future::Future, marker::PhantomData, net, pin::Pin, rc::Rc,
-};
+use std::{convert::TryFrom, future::Future, marker::PhantomData, pin::Pin, rc::Rc, time};
 
-use bytes::{Bytes, BytesMut};
 use h2::server::{Connection, SendResponse};
 use h2::SendStream;
-use http::header::{HeaderValue, CONNECTION, CONTENT_LENGTH, DATE, TRANSFER_ENCODING};
 use log::{error, trace};
 
-use crate::codec::{AsyncRead, AsyncWrite};
 use crate::http::body::{BodySize, MessageBody, ResponseBody};
 use crate::http::config::{DateService, DispatcherConfig};
 use crate::http::error::{DispatchError, ResponseError};
-use crate::http::helpers::DataFactory;
-use crate::http::message::ResponseHead;
-use crate::http::payload::Payload;
-use crate::http::request::Request;
-use crate::http::response::Response;
-use crate::rt::time::{Instant, Sleep};
-use crate::Service;
+use crate::http::header::{
+    HeaderValue, CONNECTION, CONTENT_LENGTH, DATE, TRANSFER_ENCODING,
+};
+use crate::http::{
+    message::ResponseHead, payload::Payload, request::Request, response::Response,
+};
+use crate::io::{Filter, Io, IoRef};
+use crate::service::Service;
+use crate::time::{now, Sleep};
+use crate::util::{Bytes, BytesMut};
 
 const CHUNK_SIZE: usize = 16_384;
 
 pin_project_lite::pin_project! {
     /// Dispatcher for HTTP/2 protocol
-    pub struct Dispatcher<T, S: Service<Request = Request>, B: MessageBody, X, U> {
-        config: Rc<DispatcherConfig<T, S, X, U>>,
-        connection: Connection<T, Bytes>,
-        on_connect: Option<Box<dyn DataFactory>>,
-        peer_addr: Option<net::SocketAddr>,
-        ka_expire: Instant,
+    pub struct Dispatcher<F, S: Service<Request>, B: MessageBody, X, U> {
+        io: IoRef,
+        config: Rc<DispatcherConfig<S, X, U>>,
+        connection: Connection<Io<F>, Bytes>,
+        ka_expire: time::Instant,
         ka_timer: Option<Sleep>,
         _t: PhantomData<B>,
     }
 }
 
-impl<T, S, B, X, U> Dispatcher<T, S, B, X, U>
+impl<F, S, B, X, U> Dispatcher<F, S, B, X, U>
 where
-    T: AsyncRead + AsyncWrite + Unpin,
-    S: Service<Request = Request>,
+    F: Filter,
+    S: Service<Request>,
     S::Error: ResponseError + 'static,
     S::Response: Into<Response<B>>,
     B: MessageBody,
 {
     pub(in crate::http) fn new(
-        config: Rc<DispatcherConfig<T, S, X, U>>,
-        connection: Connection<T, Bytes>,
-        on_connect: Option<Box<dyn DataFactory>>,
+        io: IoRef,
+        config: Rc<DispatcherConfig<S, X, U>>,
+        connection: Connection<Io<F>, Bytes>,
         timeout: Option<Sleep>,
-        peer_addr: Option<net::SocketAddr>,
     ) -> Self {
         // keep-alive timer
         let (ka_expire, ka_timer) = if let Some(delay) = timeout {
-            (delay.deadline(), Some(delay))
+            let expire = config.timer.now() + config.keep_alive;
+            (expire, Some(delay))
         } else if let Some(delay) = config.keep_alive_timer() {
-            (delay.deadline(), Some(delay))
+            let expire = config.timer.now() + config.keep_alive;
+            (expire, Some(delay))
         } else {
-            (config.now(), None)
+            (now(), None)
         };
 
         Dispatcher {
+            io,
             config,
-            peer_addr,
             connection,
-            on_connect,
             ka_expire,
             ka_timer,
             _t: PhantomData,
@@ -72,10 +69,10 @@ where
     }
 }
 
-impl<T, S, B, X, U> Future for Dispatcher<T, S, B, X, U>
+impl<F, S, B, X, U> Future for Dispatcher<F, S, B, X, U>
 where
-    T: AsyncRead + AsyncWrite + Unpin,
-    S: Service<Request = Request>,
+    F: Filter,
+    S: Service<Request>,
     S::Error: ResponseError + 'static,
     S::Future: 'static,
     S::Response: Into<Response<B>> + 'static,
@@ -111,12 +108,7 @@ where
                     head.method = parts.method;
                     head.version = parts.version;
                     head.headers = parts.headers.into();
-                    head.peer_addr = this.peer_addr;
-
-                    // set on_connect data
-                    if let Some(ref on_connect) = this.on_connect {
-                        on_connect.set(&mut req.extensions_mut());
-                    }
+                    head.io = Some(this.io.clone());
 
                     crate::rt::spawn(ServiceResponse {
                         state: ServiceResponseState::ServiceCall {
@@ -237,8 +229,7 @@ where
 
                         let mut send = send.take().unwrap();
                         let mut size = body.size();
-                        let h2_res =
-                            self.as_mut().prepare_response(res.head(), &mut size);
+                        let h2_res = self.as_mut().prepare_response(res.head(), &mut size);
                         this = self.as_mut().project();
 
                         let stream = match send.send_response(h2_res, size.is_eof()) {
@@ -264,8 +255,7 @@ where
 
                         let mut send = send.take().unwrap();
                         let mut size = body.size();
-                        let h2_res =
-                            self.as_mut().prepare_response(res.head(), &mut size);
+                        let h2_res = self.as_mut().prepare_response(res.head(), &mut size);
                         this = self.as_mut().project();
 
                         let stream = match send.send_response(h2_res, size.is_eof()) {
