@@ -945,8 +945,7 @@ mod tests {
     #[test]
     fn test_serde_json_in_body() {
         use serde_json::json;
-        let resp =
-            Response::build(StatusCode::OK).body(json!({"test-key":"test-value"}));
+        let resp = Response::build(StatusCode::OK).body(json!({"test-key":"test-value"}));
         assert_eq!(resp.body().get_ref(), br#"{"test-key":"test-value"}"#);
     }
 
