@@ -1,71 +1,66 @@
-use std::{
-    cell, error, fmt, future::Future, marker, net, pin::Pin, rc::Rc, task::Context,
-    task::Poll,
-};
+use std::task::{Context, Poll};
+use std::{cell, error, fmt, future, marker, pin::Pin, rc::Rc};
 
 use h2::server::{self, Handshake};
+use ntex_tls::types::HttpProtocol;
 
-use crate::codec::{AsyncRead, AsyncWrite};
-use crate::framed::State;
-use crate::rt::net::TcpStream;
-use crate::service::{pipeline_factory, IntoServiceFactory, Service, ServiceFactory};
+use crate::io::{types, Filter, Io, IoRef};
+use crate::service::{IntoServiceFactory, Service, ServiceFactory};
 use crate::time::{Millis, Seconds};
-use crate::util::{Bytes, Pool, PoolId};
+use crate::util::Bytes;
 
 use super::body::MessageBody;
 use super::builder::HttpServiceBuilder;
 use super::config::{DispatcherConfig, KeepAlive, OnRequest, ServiceConfig};
 use super::error::{DispatchError, ResponseError};
-use super::helpers::DataFactory;
 use super::request::Request;
 use super::response::Response;
-use super::{h1, h2::Dispatcher, Protocol};
+use super::{h1, h2::Dispatcher};
 
 /// `ServiceFactory` HTTP1.1/HTTP2 transport implementation
-pub struct HttpService<T, S, B, X = h1::ExpectHandler, U = h1::UpgradeHandler<T>> {
+pub struct HttpService<F, S, B, X = h1::ExpectHandler, U = h1::UpgradeHandler<F>> {
     srv: S,
     cfg: ServiceConfig,
     expect: X,
     upgrade: Option<U>,
-    on_connect: Option<Rc<dyn Fn(&T) -> Box<dyn DataFactory>>>,
-    on_request: cell::RefCell<Option<OnRequest<T>>>,
-    _t: marker::PhantomData<(T, B)>,
+    on_request: cell::RefCell<Option<OnRequest>>,
+    _t: marker::PhantomData<(F, B)>,
 }
 
-impl<T, S, B> HttpService<T, S, B>
+impl<F, S, B> HttpService<F, S, B>
 where
-    S: ServiceFactory<Config = (), Request = Request>,
+    S: ServiceFactory<Request>,
     S::Error: ResponseError + 'static,
     S::InitError: fmt::Debug,
     S::Response: Into<Response<B>> + 'static,
     S::Future: 'static,
-    <S::Service as Service>::Future: 'static,
+    <S::Service as Service<Request>>::Future: 'static,
     B: MessageBody + 'static,
 {
     /// Create builder for `HttpService` instance.
-    pub fn build() -> HttpServiceBuilder<T, S> {
+    pub fn build() -> HttpServiceBuilder<F, S> {
         HttpServiceBuilder::new()
     }
 }
 
-impl<T, S, B> HttpService<T, S, B>
+impl<F, S, B> HttpService<F, S, B>
 where
-    S: ServiceFactory<Config = (), Request = Request>,
+    F: Filter,
+    S: ServiceFactory<Request>,
     S::Error: ResponseError + 'static,
     S::InitError: fmt::Debug,
     S::Response: Into<Response<B>> + 'static,
     S::Future: 'static,
-    <S::Service as Service>::Future: 'static,
+    <S::Service as Service<Request>>::Future: 'static,
     B: MessageBody + 'static,
 {
     /// Create new `HttpService` instance.
-    pub fn new<F: IntoServiceFactory<S>>(service: F) -> Self {
+    pub fn new<U: IntoServiceFactory<S, Request>>(service: U) -> Self {
         let cfg = ServiceConfig::new(
             KeepAlive::Timeout(Seconds(5)),
             Millis(5_000),
-            Seconds::ZERO,
+            Seconds::ONE,
             Millis(5_000),
-            PoolId::P1,
         );
 
         HttpService {
@@ -73,37 +68,36 @@ where
             srv: service.into_factory(),
             expect: h1::ExpectHandler,
             upgrade: None,
-            on_connect: None,
             on_request: cell::RefCell::new(None),
             _t: marker::PhantomData,
         }
     }
 
     /// Create new `HttpService` instance with config.
-    pub(crate) fn with_config<F: IntoServiceFactory<S>>(
+    pub(crate) fn with_config<U: IntoServiceFactory<S, Request>>(
         cfg: ServiceConfig,
-        service: F,
+        service: U,
     ) -> Self {
         HttpService {
             cfg,
             srv: service.into_factory(),
             expect: h1::ExpectHandler,
             upgrade: None,
-            on_connect: None,
             on_request: cell::RefCell::new(None),
             _t: marker::PhantomData,
         }
     }
 }
 
-impl<T, S, B, X, U> HttpService<T, S, B, X, U>
+impl<F, S, B, X, U> HttpService<F, S, B, X, U>
 where
-    S: ServiceFactory<Config = (), Request = Request>,
+    F: Filter,
+    S: ServiceFactory<Request>,
     S::Error: ResponseError + 'static,
     S::InitError: fmt::Debug,
     S::Response: Into<Response<B>> + 'static,
     S::Future: 'static,
-    <S::Service as Service>::Future: 'static,
+    <S::Service as Service<Request>>::Future: 'static,
     B: MessageBody,
 {
     /// Provide service for `EXPECT: 100-Continue` support.
@@ -111,19 +105,17 @@ where
     /// Service get called with request that contains `EXPECT` header.
     /// Service must return request in case of success, in that case
     /// request will be forwarded to main service.
-    pub fn expect<X1>(self, expect: X1) -> HttpService<T, S, B, X1, U>
+    pub fn expect<X1>(self, expect: X1) -> HttpService<F, S, B, X1, U>
     where
-        X1: ServiceFactory<Config = (), Request = Request, Response = Request>,
+        X1: ServiceFactory<Request, Response = Request>,
         X1::Error: ResponseError,
         X1::InitError: fmt::Debug,
-        X1::Future: 'static,
     {
         HttpService {
             expect,
             cfg: self.cfg,
             srv: self.srv,
             upgrade: self.upgrade,
-            on_connect: self.on_connect,
             on_request: self.on_request,
             _t: marker::PhantomData,
         }
@@ -133,123 +125,61 @@ where
     ///
     /// If service is provided then normal requests handling get halted
     /// and this service get called with original request and framed object.
-    pub fn upgrade<U1>(self, upgrade: Option<U1>) -> HttpService<T, S, B, X, U1>
+    pub fn upgrade<U1>(self, upgrade: Option<U1>) -> HttpService<F, S, B, X, U1>
     where
-        U1: ServiceFactory<
-            Config = (),
-            Request = (Request, T, State, h1::Codec),
-            Response = (),
-        >,
-        U1::Error: fmt::Display + error::Error + 'static,
+        U1: ServiceFactory<(Request, Io<F>, h1::Codec), Response = ()>,
+        U1::Error: fmt::Display + error::Error,
         U1::InitError: fmt::Debug,
-        U1::Future: 'static,
     {
         HttpService {
             upgrade,
             cfg: self.cfg,
             srv: self.srv,
             expect: self.expect,
-            on_connect: self.on_connect,
             on_request: self.on_request,
             _t: marker::PhantomData,
         }
     }
 
-    /// Set on connect callback.
-    pub(crate) fn on_connect(
-        mut self,
-        f: Option<Rc<dyn Fn(&T) -> Box<dyn DataFactory>>>,
-    ) -> Self {
-        self.on_connect = f;
-        self
-    }
-
     /// Set on request callback.
-    pub(crate) fn on_request(self, f: Option<OnRequest<T>>) -> Self {
+    pub(crate) fn on_request(self, f: Option<OnRequest>) -> Self {
         *self.on_request.borrow_mut() = f;
         self
     }
 }
 
-impl<S, B, X, U> HttpService<TcpStream, S, B, X, U>
-where
-    S: ServiceFactory<Config = (), Request = Request>,
-    S::Error: ResponseError + 'static,
-    S::InitError: fmt::Debug,
-    S::Response: Into<Response<B>> + 'static,
-    S::Future: 'static,
-    <S::Service as Service>::Future: 'static,
-    B: MessageBody + 'static,
-    X: ServiceFactory<Config = (), Request = Request, Response = Request>,
-    X::Error: ResponseError + 'static,
-    X::InitError: fmt::Debug,
-    X::Future: 'static,
-    <X::Service as Service>::Future: 'static,
-    U: ServiceFactory<
-        Config = (),
-        Request = (Request, TcpStream, State, h1::Codec),
-        Response = (),
-    >,
-    U::Error: fmt::Display + error::Error + 'static,
-    U::InitError: fmt::Debug,
-    U::Future: 'static,
-    <U::Service as Service>::Future: 'static,
-{
-    /// Create simple tcp stream service
-    pub fn tcp(
-        self,
-    ) -> impl ServiceFactory<
-        Config = (),
-        Request = TcpStream,
-        Response = (),
-        Error = DispatchError,
-        InitError = (),
-    > {
-        pipeline_factory(|io: TcpStream| async move {
-            let peer_addr = io.peer_addr().ok();
-            Ok((io, Protocol::Http1, peer_addr))
-        })
-        .and_then(self)
-    }
-}
-
 #[cfg(feature = "openssl")]
 mod openssl {
-    use super::*;
-    use crate::server::openssl::{Acceptor, SslAcceptor, SslStream};
-    use crate::server::SslError;
+    use ntex_tls::openssl::{Acceptor, SslFilter};
+    use tls_openssl::ssl::SslAcceptor;
 
-    impl<S, B, X, U> HttpService<SslStream<TcpStream>, S, B, X, U>
+    use super::*;
+    use crate::server::SslError;
+    use crate::service::pipeline_factory;
+
+    impl<F, S, B, X, U> HttpService<SslFilter<F>, S, B, X, U>
     where
-        S: ServiceFactory<Config = (), Request = Request>,
-        S::Error: ResponseError + 'static,
+        F: Filter,
+        S: ServiceFactory<Request> + 'static,
+        S::Error: ResponseError,
         S::InitError: fmt::Debug,
-        S::Response: Into<Response<B>> + 'static,
+        S::Response: Into<Response<B>>,
         S::Future: 'static,
-        <S::Service as Service>::Future: 'static,
         B: MessageBody + 'static,
-        X: ServiceFactory<Config = (), Request = Request, Response = Request>,
-        X::Error: ResponseError + 'static,
+        X: ServiceFactory<Request, Response = Request> + 'static,
+        X::Error: ResponseError,
         X::InitError: fmt::Debug,
         X::Future: 'static,
-        <X::Service as Service>::Future: 'static,
-        U: ServiceFactory<
-            Config = (),
-            Request = (Request, SslStream<TcpStream>, State, h1::Codec),
-            Response = (),
-        >,
-        U::Error: fmt::Display + error::Error + 'static,
+        U: ServiceFactory<(Request, Io<SslFilter<F>>, h1::Codec), Response = ()> + 'static,
+        U::Error: fmt::Display + error::Error,
         U::InitError: fmt::Debug,
-        U::Future: 'static,
-        <U::Service as Service>::Future: 'static,
     {
         /// Create openssl based service
         pub fn openssl(
             self,
             acceptor: SslAcceptor,
         ) -> impl ServiceFactory<
-            Config = (),
-            Request = TcpStream,
+            Io<F>,
             Response = (),
             Error = SslError<DispatchError>,
             InitError = (),
@@ -260,19 +190,6 @@ mod openssl {
                     .map_err(SslError::Ssl)
                     .map_init_err(|_| panic!()),
             )
-            .and_then(|io: SslStream<TcpStream>| async move {
-                let proto = if let Some(protos) = io.ssl().selected_alpn_protocol() {
-                    if protos.windows(2).any(|window| window == b"h2") {
-                        Protocol::Http2
-                    } else {
-                        Protocol::Http1
-                    }
-                } else {
-                    Protocol::Http1
-                };
-                let peer_addr = io.get_ref().peer_addr().ok();
-                Ok((io, proto, peer_addr))
-            })
             .and_then(self.map_err(SslError::Service))
         }
     }
@@ -280,41 +197,33 @@ mod openssl {
 
 #[cfg(feature = "rustls")]
 mod rustls {
-    use super::*;
-    use crate::server::rustls::{Acceptor, ServerConfig, TlsStream};
-    use crate::server::SslError;
+    use ntex_tls::rustls::{Acceptor, TlsFilter};
+    use tls_rustls::ServerConfig;
 
-    impl<S, B, X, U> HttpService<TlsStream<TcpStream>, S, B, X, U>
+    use super::*;
+    use crate::{server::SslError, service::pipeline_factory};
+
+    impl<F, S, B, X, U> HttpService<TlsFilter<F>, S, B, X, U>
     where
-        S: ServiceFactory<Config = (), Request = Request>,
-        S::Error: ResponseError + 'static,
+        F: Filter,
+        S: ServiceFactory<Request> + 'static,
+        S::Error: ResponseError,
         S::InitError: fmt::Debug,
-        S::Future: 'static,
-        S::Response: Into<Response<B>> + 'static,
-        <S::Service as Service>::Future: 'static,
+        S::Response: Into<Response<B>>,
         B: MessageBody + 'static,
-        X: ServiceFactory<Config = (), Request = Request, Response = Request>,
-        X::Error: ResponseError + 'static,
+        X: ServiceFactory<Request, Response = Request> + 'static,
+        X::Error: ResponseError,
         X::InitError: fmt::Debug,
-        X::Future: 'static,
-        <X::Service as Service>::Future: 'static,
-        U: ServiceFactory<
-            Config = (),
-            Request = (Request, TlsStream<TcpStream>, State, h1::Codec),
-            Response = (),
-        >,
-        U::Error: fmt::Display + error::Error + 'static,
+        U: ServiceFactory<(Request, Io<TlsFilter<F>>, h1::Codec), Response = ()> + 'static,
+        U::Error: fmt::Display + error::Error,
         U::InitError: fmt::Debug,
-        U::Future: 'static,
-        <U::Service as Service>::Future: 'static,
     {
         /// Create openssl based service
         pub fn rustls(
             self,
             mut config: ServerConfig,
         ) -> impl ServiceFactory<
-            Config = (),
-            Request = TcpStream,
+            Io<F>,
             Response = (),
             Error = SslError<DispatchError>,
             InitError = (),
@@ -323,70 +232,42 @@ mod rustls {
             config.alpn_protocols = protos;
 
             pipeline_factory(
-                Acceptor::new(config)
+                Acceptor::from(config)
                     .timeout(self.cfg.0.ssl_handshake_timeout)
-                    .map_err(SslError::Ssl)
+                    .map_err(|e| SslError::Ssl(Box::new(e)))
                     .map_init_err(|_| panic!()),
             )
-            .and_then(|io: TlsStream<TcpStream>| async move {
-                let proto = io
-                    .get_ref()
-                    .1
-                    .alpn_protocol()
-                    .and_then(|protos| {
-                        if protos.windows(2).any(|window| window == b"h2") {
-                            Some(Protocol::Http2)
-                        } else {
-                            None
-                        }
-                    })
-                    .unwrap_or(Protocol::Http1);
-                let peer_addr = io.get_ref().0.peer_addr().ok();
-                Ok((io, proto, peer_addr))
-            })
             .and_then(self.map_err(SslError::Service))
         }
     }
 }
 
-impl<T, S, B, X, U> ServiceFactory for HttpService<T, S, B, X, U>
+impl<F, S, B, X, U> ServiceFactory<Io<F>> for HttpService<F, S, B, X, U>
 where
-    T: AsyncRead + AsyncWrite + Unpin + 'static,
-    S: ServiceFactory<Config = (), Request = Request>,
-    S::Error: ResponseError + 'static,
+    F: Filter + 'static,
+    S: ServiceFactory<Request> + 'static,
+    S::Error: ResponseError,
     S::InitError: fmt::Debug,
-    S::Future: 'static,
-    S::Response: Into<Response<B>> + 'static,
-    <S::Service as Service>::Future: 'static,
+    S::Response: Into<Response<B>>,
     B: MessageBody + 'static,
-    X: ServiceFactory<Config = (), Request = Request, Response = Request>,
-    X::Error: ResponseError + 'static,
+    X: ServiceFactory<Request, Response = Request> + 'static,
+    X::Error: ResponseError,
     X::InitError: fmt::Debug,
-    X::Future: 'static,
-    <X::Service as Service>::Future: 'static,
-    U: ServiceFactory<
-        Config = (),
-        Request = (Request, T, State, h1::Codec),
-        Response = (),
-    >,
-    U::Error: fmt::Display + error::Error + 'static,
+    U: ServiceFactory<(Request, Io<F>, h1::Codec), Response = ()> + 'static,
+    U::Error: fmt::Display + error::Error,
     U::InitError: fmt::Debug,
-    U::Future: 'static,
-    <U::Service as Service>::Future: 'static,
 {
-    type Config = ();
-    type Request = (T, Protocol, Option<net::SocketAddr>);
     type Response = ();
     type Error = DispatchError;
     type InitError = ();
-    type Service = HttpServiceHandler<T, S::Service, B, X::Service, U::Service>;
-    type Future = Pin<Box<dyn Future<Output = Result<Self::Service, Self::InitError>>>>;
+    type Service = HttpServiceHandler<F, S::Service, B, X::Service, U::Service>;
+    type Future =
+        Pin<Box<dyn future::Future<Output = Result<Self::Service, Self::InitError>>>>;
 
     fn new_service(&self, _: ()) -> Self::Future {
         let fut = self.srv.new_service(());
         let fut_ex = self.expect.new_service(());
         let fut_upg = self.upgrade.as_ref().map(|f| f.new_service(()));
-        let on_connect = self.on_connect.clone();
         let on_request = self.on_request.borrow_mut().take();
         let cfg = self.cfg.clone();
 
@@ -408,13 +289,9 @@ where
                 None
             };
 
-            let config =
-                DispatcherConfig::new(cfg, service, expect, upgrade, on_request);
-            let pool = config.pool.into();
+            let config = DispatcherConfig::new(cfg, service, expect, upgrade, on_request);
 
             Ok(HttpServiceHandler {
-                pool,
-                on_connect,
                 config: Rc::new(config),
                 _t: marker::PhantomData,
             })
@@ -423,30 +300,27 @@ where
 }
 
 /// `Service` implementation for http transport
-pub struct HttpServiceHandler<T, S: Service, B, X: Service, U: Service> {
-    pool: Pool,
-    config: Rc<DispatcherConfig<T, S, X, U>>,
-    on_connect: Option<Rc<dyn Fn(&T) -> Box<dyn DataFactory>>>,
-    _t: marker::PhantomData<(T, B, X)>,
+pub struct HttpServiceHandler<F, S, B, X, U> {
+    config: Rc<DispatcherConfig<S, X, U>>,
+    _t: marker::PhantomData<(F, B)>,
 }
 
-impl<T, S, B, X, U> Service for HttpServiceHandler<T, S, B, X, U>
+impl<F, S, B, X, U> Service<Io<F>> for HttpServiceHandler<F, S, B, X, U>
 where
-    T: AsyncRead + AsyncWrite + Unpin + 'static,
-    S: Service<Request = Request>,
+    F: Filter + 'static,
+    S: Service<Request>,
     S::Error: ResponseError + 'static,
     S::Future: 'static,
     S::Response: Into<Response<B>> + 'static,
     B: MessageBody + 'static,
-    X: Service<Request = Request, Response = Request>,
+    X: Service<Request, Response = Request>,
     X::Error: ResponseError + 'static,
-    U: Service<Request = (Request, T, State, h1::Codec), Response = ()>,
-    U::Error: fmt::Display + error::Error + 'static,
+    U: Service<(Request, Io<F>, h1::Codec), Response = ()> + 'static,
+    U::Error: fmt::Display + error::Error,
 {
-    type Request = (T, Protocol, Option<net::SocketAddr>);
     type Response = ();
     type Error = DispatchError;
-    type Future = HttpServiceHandlerResponse<T, S, B, X, U>;
+    type Future = HttpServiceHandlerResponse<F, S, B, X, U>;
 
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
         let cfg = self.config.as_ref();
@@ -482,8 +356,6 @@ where
             ready
         };
 
-        let ready = self.pool.poll_ready(cx).is_ready() && ready;
-
         if ready {
             Poll::Ready(Ok(()))
         } else {
@@ -507,111 +379,100 @@ where
         }
     }
 
-    fn call(&self, (io, proto, peer_addr): Self::Request) -> Self::Future {
+    fn call(&self, io: Io<F>) -> Self::Future {
         log::trace!(
-            "New http connection protocol {:?} peer address {:?}",
-            proto,
-            peer_addr
+            "New http connection, peer address {:?}",
+            io.query::<types::PeerAddr>().get()
         );
-        let on_connect = self.on_connect.as_ref().map(|f| f(&io));
 
-        match proto {
-            Protocol::Http2 => HttpServiceHandlerResponse {
+        if io.query::<HttpProtocol>().get() == Some(HttpProtocol::Http2) {
+            io.set_disconnect_timeout(self.config.client_disconnect.into());
+            HttpServiceHandlerResponse {
                 state: ResponseState::H2Handshake {
                     data: Some((
+                        io.get_ref(),
                         server::Builder::new().handshake(io),
                         self.config.clone(),
-                        on_connect,
-                        peer_addr,
                     )),
                 },
-            },
-            Protocol::Http1 => HttpServiceHandlerResponse {
+            }
+        } else {
+            HttpServiceHandlerResponse {
                 state: ResponseState::H1 {
-                    fut: h1::Dispatcher::new(
-                        io,
-                        self.config.clone(),
-                        peer_addr,
-                        on_connect,
-                    ),
+                    fut: h1::Dispatcher::new(io, self.config.clone()),
                 },
-            },
+            }
         }
     }
 }
 
 pin_project_lite::pin_project! {
-    pub struct HttpServiceHandlerResponse<T, S, B, X, U>
+    pub struct HttpServiceHandlerResponse<F, S, B, X, U>
     where
-        T: AsyncRead,
-        T: AsyncWrite,
-        T: Unpin,
-        T: 'static,
-        S: Service<Request = Request>,
+        F: Filter,
+        F: 'static,
+        S: Service<Request>,
         S::Error: ResponseError,
         S::Error: 'static,
         S::Response: Into<Response<B>>,
         S::Response: 'static,
         B: MessageBody,
         B: 'static,
-        X: Service<Request = Request, Response = Request>,
+        X: Service<Request, Response = Request>,
         X::Error: ResponseError,
         X::Error: 'static,
-        U: Service<Request = (Request, T, State, h1::Codec), Response = ()>,
+        U: Service<(Request, Io<F>, h1::Codec), Response = ()>,
         U::Error: fmt::Display,
         U::Error: error::Error,
-        U::Error: 'static,
+        U: 'static,
     {
         #[pin]
-        state: ResponseState<T, S, B, X, U>,
+        state: ResponseState<F, S, B, X, U>,
     }
 }
 
 pin_project_lite::pin_project! {
     #[project = StateProject]
-    enum ResponseState<T, S, B, X, U>
+    enum ResponseState<F, S, B, X, U>
     where
-        S: Service<Request = Request>,
+        S: Service<Request>,
         S::Error: ResponseError,
         S::Error: 'static,
-        T: AsyncRead,
-        T: AsyncWrite,
-        T: Unpin,
-        T: 'static,
+        F: Filter,
+        F: 'static,
         B: MessageBody,
-        X: Service<Request = Request, Response = Request>,
+        X: Service<Request, Response = Request>,
         X::Error: ResponseError,
         X::Error: 'static,
-        U: Service<Request = (Request, T, State, h1::Codec), Response = ()>,
+        U: Service<(Request, Io<F>, h1::Codec), Response = ()>,
         U::Error: fmt::Display,
         U::Error: error::Error,
-        U::Error: 'static,
+        U: 'static,
     {
-        H1 { #[pin] fut: h1::Dispatcher<T, S, B, X, U> },
-        H2 { fut: Dispatcher<T, S, B, X, U> },
+        H1 { #[pin] fut: h1::Dispatcher<F, S, B, X, U> },
+        H2 { fut: Dispatcher<F, S, B, X, U> },
         H2Handshake { data:
-            Option<(
-                Handshake<T, Bytes>,
-                Rc<DispatcherConfig<T, S, X, U>>,
-                Option<Box<dyn DataFactory>>,
-                Option<net::SocketAddr>,
+                      Option<(
+                          IoRef,
+                Handshake<Io<F>, Bytes>,
+                Rc<DispatcherConfig<S, X, U>>,
             )>,
         },
     }
 }
 
-impl<T, S, B, X, U> Future for HttpServiceHandlerResponse<T, S, B, X, U>
+impl<F, S, B, X, U> future::Future for HttpServiceHandlerResponse<F, S, B, X, U>
 where
-    T: AsyncRead + AsyncWrite + Unpin + 'static,
-    S: Service<Request = Request>,
+    F: Filter + 'static,
+    S: Service<Request>,
     S::Error: ResponseError + 'static,
     S::Future: 'static,
     S::Response: Into<Response<B>> + 'static,
     B: MessageBody,
-    X: Service<Request = Request, Response = Request>,
+    X: Service<Request, Response = Request>,
     X::Error: ResponseError + 'static,
-    U: Service<Request = (Request, T, State, h1::Codec), Response = ()>,
-    U::Error: fmt::Display + error::Error + 'static,
+    U: Service<(Request, Io<F>, h1::Codec), Response = ()> + 'static,
+    U::Error: fmt::Display + error::Error,
 {
     type Output = Result<(), DispatchError>;
 
@@ -623,7 +484,7 @@ where
             StateProject::H2 { ref mut fut } => Pin::new(fut).poll(cx),
             StateProject::H2Handshake { data } => {
                 let conn = if let Some(ref mut item) = data {
-                    match Pin::new(&mut item.0).poll(cx) {
+                    match Pin::new(&mut item.1).poll(cx) {
                         Poll::Ready(Ok(conn)) => conn,
                         Poll::Ready(Err(err)) => {
                             trace!("H2 handshake error: {}", err);
@@ -634,9 +495,9 @@ where
                 } else {
                     panic!()
                 };
-                let (_, cfg, on_connect, peer_addr) = data.take().unwrap();
+                let (io, _, cfg) = data.take().unwrap();
                 self.as_mut().project().state.set(ResponseState::H2 {
-                    fut: Dispatcher::new(cfg, conn, on_connect, None, peer_addr),
+                    fut: Dispatcher::new(io, cfg, conn, None),
                 });
                 self.poll(cx)
             }
