@@ -6,7 +6,6 @@
 //! * `rustls` - enables ssl support via `rustls` crate
 //! * `compress` - enables compression support in http and web modules
 //! * `cookie` - enables cookie support in http and web modules
-
 #![warn(
     rust_2018_idioms,
     unreachable_pub,
@@ -19,7 +18,8 @@
     clippy::borrow_interior_mutable_const,
     clippy::needless_doctest_main,
     clippy::too_many_arguments,
-    clippy::new_without_default
+    clippy::new_without_default,
+    clippy::return_self_not_must_use
 )]
 
 #[macro_use]
@@ -33,27 +33,22 @@ pub use ntex_macros::{rt_main as main, rt_test as test};
 #[cfg(test)]
 pub(crate) use ntex_macros::rt_test2 as rt_test;
 
-pub mod channel;
 pub mod connect;
-pub mod framed;
-#[cfg(feature = "http-framework")]
 pub mod http;
 pub mod server;
-pub mod testing;
-pub mod time;
 pub mod util;
-#[cfg(feature = "http-framework")]
 pub mod web;
 pub mod ws;
 
 pub use self::service::{
     apply_fn, boxed, fn_factory, fn_factory_with_config, fn_service, into_service,
-    pipeline, pipeline_factory, IntoService, IntoServiceFactory, Service,
-    ServiceFactory, Transform,
+    pipeline, pipeline_factory, IntoService, IntoServiceFactory, Service, ServiceFactory,
+    Transform,
 };
 
 pub use futures_core::stream::Stream;
 pub use futures_sink::Sink;
+pub use ntex_util::channel;
 pub use ntex_util::task;
 
 pub mod codec {
@@ -74,3 +69,25 @@ pub mod rt {
 pub mod service {
     pub use ntex_service::*;
 }
+
+pub mod time {
+    //! Utilities for tracking time.
+    pub use ntex_util::time::*;
+}
+
+pub mod io {
+    //! IO streaming utilities.
+    pub use ntex_io::*;
+}
+
+pub mod testing {
+    //! IO testing utilities.
+    #[doc(hidden)]
+    pub use ntex_io::testing::IoTest as Io;
+    pub use ntex_io::testing::IoTest;
+}
+
+pub mod tls {
+    //! TLS support for ntex ecosystem.
+    pub use ntex_tls::*;
+}
