@@ -2,20 +2,20 @@
 //!
 //! If the response does not complete within the specified timeout, the response
 //! will be aborted.
-use std::{fmt, future::Future, marker, pin::Pin, task::Context, task::Poll, time};
+use std::{
+    fmt, future::Future, marker, marker::PhantomData, pin::Pin, task::Context, task::Poll,
+};
 
-use crate::rt::time::{sleep, Sleep};
 use crate::service::{IntoService, Service, Transform};
-use crate::util::{Either, Ready};
-
-const ZERO: time::Duration = time::Duration::from_millis(0);
+use crate::time::{sleep, Millis, Sleep};
+use crate::util::Either;
 
 /// Applies a timeout to requests.
 ///
 /// Timeout transform is disabled if timeout is set to 0
 #[derive(Debug)]
 pub struct Timeout<E = ()> {
-    timeout: time::Duration,
+    timeout: Millis,
     _t: marker::PhantomData<E>,
 }
 
@@ -66,37 +66,32 @@ impl<E: PartialEq> PartialEq for TimeoutError<E> {
     }
 }
 
-impl<E> Timeout<E> {
-    pub fn new(timeout: time::Duration) -> Self {
+impl Timeout {
+    pub fn new<T: Into<Millis>>(timeout: T) -> Self {
         Timeout {
-            timeout,
+            timeout: timeout.into(),
             _t: marker::PhantomData,
         }
     }
 }
 
-impl<E> Clone for Timeout<E> {
+impl Clone for Timeout {
     fn clone(&self) -> Self {
-        Timeout::new(self.timeout)
+        Timeout {
+            timeout: self.timeout,
+            _t: marker::PhantomData,
+        }
     }
 }
 
-impl<S, E> Transform<S> for Timeout<E>
-where
-    S: Service,
-{
-    type Request = S::Request;
-    type Response = S::Response;
-    type Error = TimeoutError<S::Error>;
-    type InitError = E;
-    type Transform = TimeoutService<S>;
-    type Future = Ready<Self::Transform, Self::InitError>;
+impl<S> Transform<S> for Timeout {
+    type Service = TimeoutService<S>;
 
-    fn new_transform(&self, service: S) -> Self::Future {
-        Ready::Ok(TimeoutService {
+    fn new_transform(&self, service: S) -> Self::Service {
+        TimeoutService {
             service,
             timeout: self.timeout,
-        })
+        }
     }
 }
 
@@ -104,32 +99,30 @@ where
 #[derive(Debug, Clone)]
 pub struct TimeoutService<S> {
     service: S,
-    timeout: time::Duration,
+    timeout: Millis,
 }
 
-impl<S> TimeoutService<S>
-where
-    S: Service,
-{
-    pub fn new<U>(timeout: time::Duration, service: U) -> Self
+impl<S> TimeoutService<S> {
+    pub fn new<T, U, R>(timeout: T, service: U) -> Self
     where
-        U: IntoService<S>,
+        T: Into<Millis>,
+        S: Service<R>,
+        U: IntoService<S, R>,
     {
         TimeoutService {
-            timeout,
+            timeout: timeout.into(),
             service: service.into_service(),
         }
     }
 }
 
-impl<S> Service for TimeoutService<S>
+impl<S, R> Service<R> for TimeoutService<S>
 where
-    S: Service,
+    S: Service<R>,
 {
-    type Request = S::Request;
     type Response = S::Response;
     type Error = TimeoutError<S::Error>;
-    type Future = Either<TimeoutServiceResponse<S>, TimeoutServiceResponse2<S>>;
+    type Future = Either<TimeoutServiceResponse<S, R>, TimeoutServiceResponse2<S, R>>;
 
     #[inline]
     fn poll_ready(&self, cx: &mut Context<'_>) -> Poll<Result<(), Self::Error>> {
@@ -141,15 +134,17 @@ where
         self.service.poll_shutdown(cx, is_error)
     }
 
-    fn call(&self, request: S::Request) -> Self::Future {
-        if self.timeout == ZERO {
+    fn call(&self, request: R) -> Self::Future {
+        if self.timeout.is_zero() {
             Either::Right(TimeoutServiceResponse2 {
                 fut: self.service.call(request),
+                _t: PhantomData,
             })
         } else {
             Either::Left(TimeoutServiceResponse {
                 fut: self.service.call(request),
-                sleep: Box::pin(sleep(self.timeout)),
+                sleep: sleep(self.timeout),
+                _t: PhantomData,
             })
         }
     }
@@ -159,21 +154,22 @@ pin_project_lite::pin_project! {
     /// `TimeoutService` response future
     #[doc(hidden)]
     #[derive(Debug)]
-    pub struct TimeoutServiceResponse<T: Service> {
+    pub struct TimeoutServiceResponse<T: Service<R>, R> {
         #[pin]
         fut: T::Future,
-        sleep: Pin<Box<Sleep>>,
+        sleep: Sleep,
+        _t: PhantomData<R>
     }
 }
 
-impl<T> Future for TimeoutServiceResponse<T>
+impl<T, R> Future for TimeoutServiceResponse<T, R>
 where
-    T: Service,
+    T: Service<R>,
 {
     type Output = Result<T::Response, TimeoutError<T::Error>>;
 
     fn poll(self: Pin<&mut Self>, cx: &mut Context<'_>) -> Poll<Self::Output> {
-        let mut this = self.project();
+        let this = self.project();
 
         // First, try polling the future
         match this.fut.poll(cx) {
@@ -183,7 +179,7 @@ where
         }
 
         // Now check the sleep
-        match Pin::new(&mut this.sleep).poll(cx) {
+        match this.sleep.poll_elapsed(cx) {
             Poll::Pending => Poll::Pending,
             Poll::Ready(_) => Poll::Ready(Err(TimeoutError::Timeout)),
         }
@@ -194,15 +190,16 @@ pin_project_lite::pin_project! {
     /// `TimeoutService` response future
     #[doc(hidden)]
     #[derive(Debug)]
-    pub struct TimeoutServiceResponse2<T: Service> {
+    pub struct TimeoutServiceResponse2<T: Service<R>, R> {
         #[pin]
         fut: T::Future,
+        _t: PhantomData<R>,
     }
 }
 
-impl<T> Future for TimeoutServiceResponse2<T>
+impl<T, R> Future for TimeoutServiceResponse2<T, R>
 where
-    T: Service,
+    T: Service<R>,
 {
     type Output = Result<T::Response, TimeoutError<T::Error>>;
 
@@ -231,8 +228,7 @@ mod tests {
     #[derive(Clone, Debug, Display, PartialEq)]
     struct SrvError;
 
-    impl Service for SleepService {
-        type Request = ();
+    impl Service<()> for SleepService {
         type Response = ();
         type Error = SrvError;
         type Future = Pin<Box<dyn Future<Output = Result<(), SrvError>>>>;
@@ -250,7 +246,7 @@ mod tests {
         }
 
         fn call(&self, _: ()) -> Self::Future {
-            let fut = crate::rt::time::sleep(self.0);
+            let fut = crate::time::sleep(self.0);
             Box::pin(async move {
                 let _ = fut.await;
                 Ok::<_, SrvError>(())
