@@ -2,7 +2,7 @@ use std::{
     cell::RefCell, fmt, marker::PhantomData, pin::Pin, rc::Rc, task::Context, task::Poll,
 };
 
-use super::{Codec, Frame, Message, ProtocolError};
+use super::{error::ProtocolError, Codec, Frame, Message};
 use crate::util::{Bytes, BytesMut};
 use crate::{codec::Decoder, codec::Encoder, Sink, Stream};
 
@@ -169,9 +169,7 @@ where
 #[cfg(test)]
 mod tests {
     use super::*;
-    use crate::{
-        channel::mpsc, util::next, util::poll_fn, util::send, util::ByteString,
-    };
+    use crate::{channel::mpsc, util::next, util::poll_fn, util::send, util::ByteString};
 
     #[crate::rt_test]
     async fn test_decoder() {
