@@ -3,7 +3,9 @@ use std::sync::atomic::{AtomicUsize, Ordering};
 use std::sync::Arc;
 
 use futures::future::ok;
-use open_ssl::ssl::{SslAcceptor, SslConnector, SslFiletype, SslMethod, SslVerifyMode};
+use tls_openssl::ssl::{
+    AlpnError, SslAcceptor, SslConnector, SslFiletype, SslMethod, SslVerifyMode,
+};
 
 use ntex::http::client::{Client, Connector};
 use ntex::http::test::server as test_server;
@@ -26,7 +28,7 @@ fn ssl_acceptor() -> SslAcceptor {
         if protos.windows(3).any(|window| window == H2) {
             Ok(b"h2")
         } else {
-            Err(open_ssl::ssl::AlpnError::NOACK)
+            Err(AlpnError::NOACK)
         }
     });
     builder.set_alpn_protos(b"\x02h2").unwrap();
@@ -48,8 +50,7 @@ async fn test_connection_reuse_h2() {
             HttpService::build()
                 .h2(map_config(
                     App::new().service(
-                        web::resource("/")
-                            .route(web::to(|| async { HttpResponse::Ok() })),
+                        web::resource("/").route(web::to(|| async { HttpResponse::Ok() })),
                     ),
                     |_| AppConfig::default(),
                 ))
@@ -66,11 +67,16 @@ async fn test_connection_reuse_h2() {
         .map_err(|e| log::error!("Cannot set alpn protocol: {:?}", e));
 
     let client = Client::build()
-        .connector(Connector::default().openssl(builder.build()).finish())
+        .connector(
+            Connector::default()
+                .timeout(Seconds(30))
+                .openssl(builder.build())
+                .finish(),
+        )
         .finish();
 
     // req 1
-    let request = client.get(srv.surl("/")).timeout(Seconds(10)).send();
+    let request = client.get(srv.surl("/")).timeout(Seconds(30)).send();
     let response = request.await.unwrap();
     assert!(response.status().is_success());
 
