@@ -3,15 +3,18 @@ use std::io;
 
 use futures::future::{err, ok, ready};
 use futures::stream::{once, Stream, StreamExt};
-use open_ssl::ssl::{AlpnError, SslAcceptor, SslFiletype, SslMethod};
+use tls_openssl::ssl::{AlpnError, SslAcceptor, SslFiletype, SslMethod};
 
+use ntex::codec::BytesCodec;
 use ntex::http::error::PayloadError;
 use ntex::http::header::{self, HeaderName, HeaderValue};
 use ntex::http::test::server as test_server;
-use ntex::http::{body, HttpService, Method, Request, Response, StatusCode, Version};
+use ntex::http::{body, h1, HttpService, Method, Request, Response, StatusCode, Version};
+use ntex::io::Io;
 use ntex::service::{fn_service, ServiceFactory};
-use ntex::util::{Bytes, BytesMut};
-use ntex::{time::Seconds, web::error::InternalError};
+use ntex::util::{Bytes, BytesMut, Ready};
+use ntex::ws::handshake_response;
+use ntex::{time::Seconds, web::error::InternalError, ws};
 
 async fn load_body<S>(stream: S) -> Result<BytesMut, PayloadError>
 where
@@ -404,10 +407,7 @@ async fn test_h2_service_error() {
     let mut srv = test_server(move || {
         HttpService::build()
             .h2(|_| {
-                err::<Response, _>(InternalError::default(
-                    "error",
-                    StatusCode::BAD_REQUEST,
-                ))
+                err::<Response, _>(InternalError::default("error", StatusCode::BAD_REQUEST))
             })
             .openssl(ssl_acceptor())
             .map_err(|_| ())
@@ -421,23 +421,6 @@ async fn test_h2_service_error() {
     assert_eq!(bytes, Bytes::from_static(b"error"));
 }
 
-#[ntex::test]
-async fn test_h2_on_connect() {
-    let srv = test_server(move || {
-        HttpService::build()
-            .on_connect(|_| 10usize)
-            .h2(|req: Request| {
-                assert!(req.extensions().contains::<usize>());
-                ok::<_, io::Error>(Response::Ok().finish())
-            })
-            .openssl(ssl_acceptor())
-            .map_err(|_| ())
-    });
-
-    let response = srv.srequest(Method::GET, "/").send().await.unwrap();
-    assert!(response.status().is_success());
-}
-
 #[ntex::test]
 async fn test_ssl_handshake_timeout() {
     use std::io::Read;
@@ -455,3 +438,50 @@ async fn test_ssl_handshake_timeout() {
     let _ = stream.read_to_string(&mut data);
     assert!(data.is_empty());
 }
+
+#[ntex::test]
+async fn test_ws_transport() {
+    let mut srv = test_server(|| {
+        HttpService::build()
+            .upgrade(|(req, io, codec): (Request, Io<_>, h1::Codec)| {
+                async move {
+                    let res = handshake_response(req.head()).finish();
+
+                    // send handshake respone
+                    io.encode(
+                        h1::Message::Item((res.drop_body(), body::BodySize::None)),
+                        &codec,
+                    )
+                    .unwrap();
+
+                    let io = io
+                        .add_filter(ws::WsTransportFactory::new(ws::Codec::default()))
+                        .await?;
+
+                    // start websocket service
+                    loop {
+                        if let Some(item) =
+                            io.recv(&BytesCodec).await.map_err(|e| e.into_inner())?
+                        {
+                            io.send(item.freeze(), &BytesCodec).await.unwrap()
+                        } else {
+                            break;
+                        }
+                    }
+                    Ok::<_, io::Error>(())
+                }
+            })
+            .finish(|_| Ready::Ok::<_, io::Error>(Response::NotFound()))
+            .openssl(ssl_acceptor())
+    });
+
+    let io = srv.wss().await.unwrap().into_inner().0;
+    let codec = ws::Codec::default().client_mode();
+
+    io.send(ws::Message::Binary(Bytes::from_static(b"text")), &codec)
+        .await
+        .unwrap();
+
+    let item = io.recv(&codec).await.unwrap().unwrap();
+    assert_eq!(item, ws::Frame::Binary(Bytes::from_static(b"text")));
+}
