@@ -353,14 +353,14 @@ async fn test_body_br_streaming() {
 
 #[ntex::test]
 async fn test_head_binary() {
-    let srv =
-        test::server_with(test::config().h1(), || {
-            App::new().service(web::resource("/").route(
-                web::head().to(move || async {
+    let srv = test::server_with(test::config().h1(), || {
+        App::new().service(
+            web::resource("/")
+                .route(web::head().to(move || async {
                     HttpResponse::Ok().content_length(100).body(STR)
-                }),
-            ))
-        });
+                })),
+        )
+    });
 
     let mut response = srv.head("/").send().await.unwrap();
     assert!(response.status().is_success());
@@ -711,7 +711,8 @@ async fn test_brotli_encoding_large() {
 #[ntex::test]
 async fn test_brotli_encoding_large_openssl() {
     // load ssl keys
-    use open_ssl::ssl::{SslAcceptor, SslFiletype, SslMethod};
+    use tls_openssl::ssl::{SslAcceptor, SslFiletype, SslMethod};
+
     let mut builder = SslAcceptor::mozilla_intermediate(SslMethod::tls()).unwrap();
     builder
         .set_private_key_file("./tests/key.pem", SslFiletype::PEM)
@@ -752,7 +753,8 @@ async fn test_brotli_encoding_large_openssl() {
 #[ntex::test]
 async fn test_brotli_encoding_large_openssl_h1() {
     // load ssl keys
-    use open_ssl::ssl::{SslAcceptor, SslFiletype, SslMethod};
+    use tls_openssl::ssl::{SslAcceptor, SslFiletype, SslMethod};
+
     let mut builder = SslAcceptor::mozilla_intermediate(SslMethod::tls()).unwrap();
     builder
         .set_private_key_file("./tests/key.pem", SslFiletype::PEM)
@@ -762,14 +764,13 @@ async fn test_brotli_encoding_large_openssl_h1() {
         .unwrap();
 
     let data = STR.repeat(10);
-    let srv =
-        test::server_with(test::config().openssl(builder.build()).h1(), move || {
-            App::new().service(web::resource("/").route(web::to(|bytes: Bytes| async {
-                HttpResponse::Ok()
-                    .encoding(ContentEncoding::Identity)
-                    .body(bytes)
-            })))
-        });
+    let srv = test::server_with(test::config().openssl(builder.build()).h1(), move || {
+        App::new().service(web::resource("/").route(web::to(|bytes: Bytes| async {
+            HttpResponse::Ok()
+                .encoding(ContentEncoding::Identity)
+                .body(bytes)
+        })))
+    });
 
     // body
     let mut e = BrotliEncoder::new(Vec::new(), 3);
@@ -794,7 +795,8 @@ async fn test_brotli_encoding_large_openssl_h1() {
 #[ntex::test]
 async fn test_brotli_encoding_large_openssl_h2() {
     // load ssl keys
-    use open_ssl::ssl::{AlpnError, SslAcceptor, SslFiletype, SslMethod};
+    use tls_openssl::ssl::{AlpnError, SslAcceptor, SslFiletype, SslMethod};
+
     let mut builder = SslAcceptor::mozilla_intermediate(SslMethod::tls()).unwrap();
     builder
         .set_private_key_file("./tests/key.pem", SslFiletype::PEM)
@@ -816,14 +818,13 @@ async fn test_brotli_encoding_large_openssl_h2() {
     builder.set_alpn_protos(b"\x08http/1.1\x02h2").unwrap();
 
     let data = STR.repeat(10);
-    let srv =
-        test::server_with(test::config().openssl(builder.build()).h2(), move || {
-            App::new().service(web::resource("/").route(web::to(|bytes: Bytes| async {
-                HttpResponse::Ok()
-                    .encoding(ContentEncoding::Identity)
-                    .body(bytes)
-            })))
-        });
+    let srv = test::server_with(test::config().openssl(builder.build()).h2(), move || {
+        App::new().service(web::resource("/").route(web::to(|bytes: Bytes| async {
+            HttpResponse::Ok()
+                .encoding(ContentEncoding::Identity)
+                .body(bytes)
+        })))
+    });
 
     // body
     let mut e = BrotliEncoder::new(Vec::new(), 3);
@@ -847,10 +848,10 @@ async fn test_brotli_encoding_large_openssl_h2() {
 #[cfg(all(feature = "rustls", feature = "openssl"))]
 #[ntex::test]
 async fn test_reading_deflate_encoding_large_random_rustls() {
-    use rust_tls::{Certificate, PrivateKey, ServerConfig};
-    use rustls_pemfile::{certs, pkcs8_private_keys};
-    use std::fs::File;
-    use std::io::BufReader;
+    use std::{fs::File, io::BufReader};
+
+    use rustls_pemfile::{certs, rsa_private_keys};
+    use tls_rustls::{Certificate, PrivateKey, ServerConfig};
 
     let data = rand::thread_rng()
         .sample_iter(&Alphanumeric)
@@ -866,7 +867,7 @@ async fn test_reading_deflate_encoding_large_random_rustls() {
         .iter()
         .map(|c| Certificate(c.to_vec()))
         .collect();
-    let keys = PrivateKey(pkcs8_private_keys(key_file).unwrap().remove(0));
+    let keys = PrivateKey(rsa_private_keys(key_file).unwrap().remove(0));
     let config = ServerConfig::builder()
         .with_safe_defaults()
         .with_no_client_auth()
@@ -905,10 +906,10 @@ async fn test_reading_deflate_encoding_large_random_rustls() {
 #[cfg(all(feature = "rustls", feature = "openssl"))]
 #[ntex::test]
 async fn test_reading_deflate_encoding_large_random_rustls_h1() {
-    use rust_tls::{Certificate, PrivateKey, ServerConfig};
-    use rustls_pemfile::{certs, pkcs8_private_keys};
+    use rustls_pemfile::{certs, rsa_private_keys};
     use std::fs::File;
     use std::io::BufReader;
+    use tls_rustls::{Certificate, PrivateKey, ServerConfig};
 
     let data = rand::thread_rng()
         .sample_iter(&Alphanumeric)
@@ -924,7 +925,7 @@ async fn test_reading_deflate_encoding_large_random_rustls_h1() {
         .iter()
         .map(|c| Certificate(c.to_vec()))
         .collect();
-    let keys = PrivateKey(pkcs8_private_keys(key_file).unwrap().remove(0));
+    let keys = PrivateKey(rsa_private_keys(key_file).unwrap().remove(0));
     let config = ServerConfig::builder()
         .with_safe_defaults()
         .with_no_client_auth()
@@ -963,10 +964,10 @@ async fn test_reading_deflate_encoding_large_random_rustls_h1() {
 #[cfg(all(feature = "rustls", feature = "openssl"))]
 #[ntex::test]
 async fn test_reading_deflate_encoding_large_random_rustls_h2() {
-    use rust_tls::{Certificate, PrivateKey, ServerConfig};
-    use rustls_pemfile::{certs, pkcs8_private_keys};
-    use std::fs::File;
-    use std::io::BufReader;
+    use std::{fs::File, io::BufReader};
+
+    use rustls_pemfile::{certs, rsa_private_keys};
+    use tls_rustls::{Certificate, PrivateKey, ServerConfig};
 
     let data = rand::thread_rng()
         .sample_iter(&Alphanumeric)
@@ -982,7 +983,7 @@ async fn test_reading_deflate_encoding_large_random_rustls_h2() {
         .iter()
         .map(|c| Certificate(c.to_vec()))
         .collect();
-    let keys = PrivateKey(pkcs8_private_keys(key_file).unwrap().remove(0));
+    let keys = PrivateKey(rsa_private_keys(key_file).unwrap().remove(0));
     let config = ServerConfig::builder()
         .with_safe_defaults()
         .with_no_client_auth()
